/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 library for SPB module M-COM-8                 **/
/**                                                                       **/
/**                             Version 1.B.                              **/
/**                                                                       **/
/**  This file contains routines for Borland C++/3.1 to allow the user,   **/
/**  to write programs for the MODULAR-4 in "C"                           **/
/**                                                                       **/
/**             Project start    : 01.07.1995                             **/
/**             Last change      : 11.12.1995                             **/
/**                                                                       **/
/**                         Author : A.H.                                 **/
/**                         Status : o.k.                                 **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/** This library was tested with Borland C++ Version 3.1.                 **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1995                           **/
/**                                                                       **/
/***************************************************************************/

/* The switches for the library are set in the configuration files of Pascal
   (*.TP file) und C (project-file). They can also be set in this source file:

   Then the following #defines have to exist:

   If you use Pascal, #define BORLAND_PASCAL has to be included (default is C).

   If the library shall be used as realtime library on the MODULAR-4/486
   #define ML8 and #define ML8RT have to be included (default is PC library).

   If you work with Windows, this is detected by the compiler. No #defines have
   to be included.
*/

#ifdef ML8RT
/* As the parameters micro_slot und offset have changed in the function
   ml8rt_read_eeprom, using ml8rtbib version 1.C. and less the instruction
   #define ML8RT_VERSION_1C has to be added
*/
#endif

#include "..\..\..\c\m048_lib.h"


#ifdef BORLAND_PASCAL

       #define STATIC static           /* for Definition of arrays         */

       #ifndef ML8
	       extern void pascal ml4_out_port(ushort port, byte data);
	       extern byte pascal ml4_in_port(ushort port);
	       extern ushort pascal ml4_read_eeprom(byte micro_slot, byte offset);
       #endif
#else
       #define STATIC static far       /* for Definition of arrays         */

       #ifndef ML8
	       #ifdef __DLL__          /* if module DLL shall be generated */
		    #include "WML4BIB.H"
		    #include "WML4DEB.H"
	       #else
		    #include "ML4BIB.H"
		    #include "ML4DEB.H"
	       #endif
       #endif
#endif

#ifdef ML8
       #define ANZ_MOD 11              /* MODULAR-4/486: number of module  */
				       /* slots: 10+1 (slot 0 is not used) */

       const STATIC ushort micro_slot_basis[ANZ_MOD] = {0, 0x400, 0x500,
							   0x600, 0x700,
							   0x420, 0x440,
							   0x460, 0x480,
							   0x4A0, 0x4C0};

       #ifndef ML8RT
	      #include "ML8BIB.H"

				       /* The functions of library ML8BIB  */
				       /* all begin with ml8_... :         */
	      #define mlx_read_eeprom            ml8_read_eeprom_copy
	      #define mlx_out_port               ml8_out8_port
	      #define mlx_in_port                ml8_in8_port
       #else
	      #include "stdio.h"
	      #include "dos.h"
	      #include "ML8RTBIB.H"

	      #define mlx_out_port(port, dat1)   outportb(port, dat1)
	      #define mlx_in_port(port)          inportb(port)
	      #define ml8_in16_port(port)        inport(port)
	      #define ml8_out16_port(port, dat1) outport(port, dat1)
       #endif
#else
       #define ANZ_MOD 5               /* MODULAR-4/Zxx: number of module  */
				       /* slots 4 + 1 (slot 0 is not used) */
       const STATIC byte micro_slot_basis[ANZ_MOD] = {0, 0x80, 0xA0,
							 0xC0, 0xE0};


				       /* The functions of library         */
				       /* ML4BIB/ML4DEB all begin with     */
				       /* ml4_... :                        */
       #define mlx_read_eeprom                   ml4_read_eeprom
       #define mlx_out_port                      ml4_out_port
       #define mlx_in_port                       ml4_in_port
#endif

#ifdef ML8RT
       #ifdef ML8RT_VERSION_1C
	      #define mlx_read_eeprom(micro_slot, offset) ml8rt_read_eeprom_copy(offset, micro_slot)
       #else
	      #define mlx_read_eeprom(micro_slot, offset) ml8rt_read_eeprom_copy(micro_slot, offset)
       #endif
#endif

 const STATIC ushort channels[8] = {0x02, 0x00,
				    0x06, 0x04,
				    0x0A, 0x08,
				    0x0E, 0x0C};

#define CHANNEL channels[channel]      /* Channels A..H                    */


				       /* Static Arrays                    */

				       /* SCC-Type :                       */
				       /*           0 = 8530               */
				       /*           1 = 85130              */
				       /*           2 = 85230              */
 STATIC ushort m048scc_type[ANZ_MOD][4];

				       /* Baseclock-Register :             */
				       /*           1 = 8530               */
				       /*           2 = 85130              */
				       /*           3 = 85230              */
 STATIC short  m048baseclk[ANZ_MOD][4];

				       /* Clock-Select-Register :          */
				       /*           0 = -                  */
				       /*           1 = Baseclk 1          */
				       /*           2 = Baseclk 2          */
				       /*           3 = Baseclk 3          */
				       /*           4 = Baseclk 4          */
				       /*           5 = Quartz / 4         */
				       /*           6 = -                  */
				       /*           7 = CTS                */
 STATIC byte   m048clksel[ANZ_MOD][8];

				       /* Interrupt-Select-Register :      */
				       /*           0 = -                  */
				       /*           1 = IRQ-A / INT-1      */
				       /*           2 = IRQ-B / INT-3      */
				       /*           3 = IRQ-E / INT-4      */
				       /*           4 = IRQ-C / INT-5      */
				       /*           5 = IRQ-D / INT-7      */

 STATIC short  m048intsel[ANZ_MOD];
				       /* PCLK-Select-Register :           */
				       /*           0 = PCLK = Quartz / 5  */
				       /*           1 = PCLK = Quartz / 4  */
				       /*           2 = PCLK = Quartz / 3  */
				       /*           3 = PCLK = Quartz / 2  */
 STATIC short  m048pclk[ANZ_MOD];

 STATIC ushort m048quartz[ANZ_MOD];    /* QUARTZ Frequency of module       */
				       /* 25000 = 25000 kHz = 25 MHz       */

 STATIC byte   tmtctr[ANZ_MOD][8];


#ifdef __DLL__                         /* if module DLL shall be generated */

#include <windows.h>

HANDLE   hInst;

#pragma argsused

int far pascal LibMain(hInstance, wDataSeg, wHeapSize, lpszCmdLine)
HANDLE  hInstance;                     /* Instance of DLL                  */
WORD    wDataSeg;                      /* Data segment                     */
WORD    wHeapSize;                     /* Size of Heap                     */
LPSTR   lpszCmdLine;                   /* Pointer to command line          */

{
 hInst = hInstance;

 if(wHeapSize > 0) UnlockData(0);

 return(1);
}

#endif  /* __DLL__ */

#ifndef BORLAND_PASCAL
#endif


/***************************************************************************/
/* Init library with EEPROM-values                                         */
/*                                                                         */
/* This procedure checks the module type in slot "micro_slot", reads the   */
/* EEPROM values and sets all registers in the module's gate-array         */
/* (PCLK-Select, Interrupt-Select, Clock-Select and the Baseclk-registers).*/
/* The register values are stored in static library-variables.             */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/***************************************************************************/
void EXPORT m048_bib_startup(byte micro_slot)
{
 ushort w;
 short  i, endm, m;

				       /* Check, whether module extender   */
				       /* is placed at slot 1              */
 if(lobyte(mlx_read_eeprom(1, 0)) == 0x29) endm = ANZ_MOD;
 else endm = 5;                        /* 4 slots                          */

 for(m = 1; m < endm; m++)             /* Check all slots                  */
   if(lobyte(mlx_read_eeprom(m, 0)) == 0x30)
     {                                 /* Right module = 48 = 0x30         */
      for(i = 0; i < 8; i++)
	{                              /* Read Clock-Select-Register       */
	 w = mlx_read_eeprom(m, 7 + i);
	 m048clksel[m][i] = (byte)w;
				       /* Set Channel for Clksel-Register  */
	 mlx_out_port(MBASIS + M048_CHANNELSEL, (byte)i);
	 mlx_out_port(MBASIS + M048_CLKSEL, (byte)w);
	}

				       /* Read/Set Baseclk-Registers       */
      w = mlx_read_eeprom(m, 15);
      m048baseclk[m][0] = (byte)w;
      mlx_out_port(MBASIS + M048_BASECLK1, (byte)w);
      w = mlx_read_eeprom(m, 16);
      m048baseclk[m][1] = (byte)w;
      mlx_out_port(MBASIS + M048_BASECLK2, (byte)w);
      w = mlx_read_eeprom(m, 17);
      m048baseclk[m][2] = (byte)w;
      mlx_out_port(MBASIS + M048_BASECLK3, (byte)w);
      w = mlx_read_eeprom(m, 18);
      m048baseclk[m][3] = (byte)w;
      mlx_out_port(MBASIS + M048_BASECLK4, (byte)w);

				       /* Read/Set Interrupt-Select-Reg.   */
      w = mlx_read_eeprom(m, 5);
      m048intsel[m] = (byte)w;
      mlx_out_port(MBASIS + M048_INTSEL, (byte)w);

				       /* Read/Set PCLK-Select-Register    */
      w = mlx_read_eeprom(m, 6);
      m048pclk[m] = (byte)w;
      mlx_out_port(MBASIS + M048_PCLKSEL, (byte)w);


      w = mlx_read_eeprom(m, 2);
      m048scc_type[m][0] = lobyte(w);
      m048scc_type[m][1] = hibyte(w);
				       /* get Hardware of SCC 3 and 4      */
      w = mlx_read_eeprom(m, 3);
      m048scc_type[m][2] = lobyte(w);
      m048scc_type[m][3] = hibyte(w);

      w = mlx_read_eeprom(m, 4);       /* get quartz frequency of module   */
      m048quartz[m]  = ((w & 0x000F))       *    1;
      m048quartz[m] += ((w & 0x00F0) >>  4) *   10;
      m048quartz[m] += ((w & 0x0F00) >>  8) *  100;
      m048quartz[m] += ((w & 0xF000) >> 12) * 1000;
      m048quartz[m] *= 10;
     }
}



/***************************************************************************/
/* Hardware-Reset SCC                                                      */
/*                                                                         */
/* This procedure forces a hardware interrupt on SCC no. "scc_num".        */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* scc_num    : SCC-Number  0..3                                           */
/***************************************************************************/
void EXPORT m048_scc_reset(byte micro_slot, byte scc_num)
{                                      /* SCC-Force Hardware-Reset         */
 mlx_out_port(MBASIS + scc_num * 4, 0x09);
 mlx_out_port(MBASIS + scc_num * 4, 0xC0);
}




/***************************************************************************/
/* Initialize one Channel of one SCC                                       */
/*                                                                         */
/* This function initializes one channel of one SCC.                       */
/* The interrupt vector is set to the SCC number (0..3) in WR 2, Bit 6..7. */
/* The parameter "int_mode" has the same function as WR 1 of a SCC,        */
/* only the lower 5 bits are used.                                         */
/* The parameter "mode" selects the clock-mode for one channel.            */
/* It is possible to connect a module-clock (PCLK, baseclk 1..4, etc.) to  */
/* generate the transmit and receive clocks, the clock can also be delive- */
/* red by another channel using the RTS-line of the interface or a clock   */
/* can be delivered to another interface via the RTS-line.                 */
/* It is important not to change the order of the SCC-register program. !  */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* mode       : Select Clock-Mode Control (=WR11)                          */
/*                7 6 5 4 3 2 1 0                                          */
/*                | | | | | | 0-0--- TRxC Out = Xtal-Output (not used !)   */
/*                | | | | | | 0-1--- TRxC Out = Transmit Clock             */
/*                | | | | | | 1-0--- TRxC Out = BRGenerator Output         */
/*                | | | | | | 1-1--- TRxC Out = DPLL Output                */
/*                | | | | | 0------- TRxC = Input                          */
/*                | | | | | 1------- TRxC = Output                         */
/*                | | | 0-0--------- Transmit Clock = RTxC Pin             */
/*                | | | 0-1--------- Transmit Clock = TRxC Pin             */
/*                | | | 1-0--------- Transmit Clock = BRGenerator Output   */
/*                | | | 1-1--------- Transmit Clock = DPLL Output          */
/*                | 0-0------------- Receive Clock  = RTxC Pin             */
/*                | 0-1------------- Receive Clock  = TRxC Pin             */
/*                | 1-0------------- Receive Clock  = BRGenerator Output   */
/*                | 1-1------------- Receive Clock  = DPLL Output          */
/*                0----------------- RTxC-Input = TTL, no Crystal Osc.     */
/*                                                                         */
/* Examples:                                                               */
/*                                                                         */
/* use mode = 0x56 if : transmit and receive clock come from a baseclk     */
/*                      and the baudrate-generator is used;                */
/*                      the TRxC pin provides the baudrate-generator output*/
/*                      e.g. as clock source for another interface via RTS;*/
/*                      this must be selected in the clock-select register */
/*                      of this channel                                    */
/* use mode = 0x05 if : transmit and receive clock come from CTS via RTxC; */
/*                      this must be selected in the clock-select register */
/*                      of this channel;                                   */
/*                      the baudrate-generator has no function             */
/*                                                                         */
/* int_mode   : =Write Register 1 of a SCCs channel (WR 1, lower 5 bits)   */
/*                 7 6 5 4 3 2 1 0                                         */
/*                 | | | | | | | 0--- Ext./Status Master Interrupt Disable */
/*                 | | | | | | | 1--- Ext./Status Master Interrupt Enable  */
/*                 | | | | | | 0----- Transmit Buffer Empty Inter. Disable */
/*                 | | | | | | 1----- Transmit Buffer Empty Inter. Enable  */
/*                 | | | | | 0------- no Interrupt on Parity not Matching  */
/*                 | | | | | 1------- Parity not Matching Is Special Cond. */
/*                 | | | 0-0--------- Receive Interupts Disabled           */
/*                 | | | 0-1--------- Receive Interupt on First Charac. or */
/*                 | | | | |          Special Condition                    */
/*                 | | | 1-0--------- Interrupt on All Receive Charac. or  */
/*                 | | |              Special Condition                    */
/*                 | | | 1-1--------- Receive Interupt on Special Condit.  */
/*                 0 0 0 ------------ reserved                             */
/*                                                                         */
/* int_x_mode : =Write Register 15 of a SCCs channel (WR 15)               */
/*               External/Status Interrupt Control                         */
/*                 7 6 5 4 3 2 1 0                                         */
/*                 | | | | | | | 0--- reserved for ESCC                    */
/*                 | | | | | | 0----- Zero Count Interrupt Disable         */
/*                 | | | | | | 1----- Zero Count Interrupt Enable          */
/*                 | | | | | 0------- SDLC Fifo Disable (only ESCC & CMOS) */
/*                 | | | | | 1------- SDLC Fifo Enable (only ESCC & CMOS)  */
/*                 | | | | 0--------- DCD (=CTS) Interrupt Disable         */
/*                 | | | | 1--------- DCD (=CTS) Interrupt Enable          */
/*                 | | | 0----------- reserved                             */
/*                 | | 0------------- CTS (=DCD) Interrupt Disable         */
/*                 | | 1------------- CTS (=DCD) Interrupt Enable          */
/*                 | 0--------------- Transmit Underrun Interrupt Disable  */
/*                 | 1--------------- Transmit Underrun Interrupt Enable   */
/*                 0----------------- Break/Abort Interrupt Disable        */
/*                 1----------------- Break/Abort Interrupt Enable         */
/*                                                                         */
/* timeconst  : SCC-Timeconst. of BRG (baudrate generator)                 */
/*                                                                         */
/* use_brg    : if use_brg == 0, the baudrate-generator is not used        */
/*                                                                         */
/* databits   : 5, 6, 7 or 8 Databits (transmit and receive)               */
/*                                                                         */
/* stopbits   : 0=no stopbits, 1=1 stopbit, 2=1,5 stopbits, 3=2 stopbits   */
/*                                                                         */
/* parity_type: 0=no parity , 1=odd parity, 2=even parity type             */
/*                                                                         */
/* Result     : if(> 0) Error                                              */
/***************************************************************************/
byte EXPORT m048_init_scc(byte micro_slot, byte channel,
			  byte mode, byte int_mode, byte int_x_mode,
			  ushort timeconst, byte use_brg, byte clkmode,
			  byte databits, byte stopbits, byte parity_type)
{
 byte usart;
 byte rcvcnt;
 ushort sccadr;
 union
   {
    byte lo_hi[2];
    ushort integer;
   }timec;

 sccadr = MBASIS + CHANNEL;

 timec.integer = timeconst;


 /**************************************************************************/
 /*      STAGE 1. Modes and Constants                                      */
 /**************************************************************************/

				       /************************************/
				       /* WR 9 Master Interrupt Control    */
				       /************************************/
 mlx_out_port(sccadr,  9);
 switch((int)(channel & 0x01))
   {                                   /* Reset Channel A of SCC           */
    case 0: mlx_out_port(sccadr, 0x80);
	    break;
				       /* Reset Channel B of SCC           */
    case 1: mlx_out_port(sccadr, 0x40);
	    break;
    default: return(1);
   }


				       /************************************/
				       /* WR 4 Transm./Rec. Misc. & Modes  */
 usart = 0;                            /************************************/

 if(parity_type > 0)
   {
    usart = usart | 1;                 /* Parity enabled = Bit 0 = 1       */
				       /* Even Parity    = Bit 1 = 1       */
    if(parity_type == 2) usart = usart | 2;
   }

 switch(stopbits)
   {                                   /* Set Stop-Bits   = Bit 3 & 2      */
    case 0: break;
				       /* 1 Stop-Bit      =     0   1      */
    case 1: usart = usart | 0x04; break;
				       /* 1,5 Stop-Bit    =     1   0      */
    case 2: usart = usart | 0x08; break;
				       /* 2 Stop-Bits     =     1   1      */
    case 3: usart = usart | 0x0C; break;

    default: return(1);
   }
				       /* Bit 5 & 4 ignored in async. mode */

 switch((int)clkmode)
   {
				       /* Clock Mode      = Bit 7 & 6      */
    case  1: usart = usart | 0x00;     /* x1   (synchron.)=     0   0      */
	     break;
    case 16: usart = usart | 0x40;     /* x16 (asynchron.)=     0   1      */
	     break;
    case 32: usart = usart | 0x80;     /* x32 (asynchron.)=     1   0      */
	     break;
    case 64: usart = usart | 0xC0;     /* x64 (asynchron.)=     1   1      */
	     break;
    default: return(1);
   }
 mlx_out_port(sccadr,  4);             /* WR 4                             */
 mlx_out_port(sccadr, usart);


				       /************************************/
				       /* WR 1 Interrupt Enable            */
				       /************************************/

 mlx_out_port(sccadr,  1);             /* Select IW/REQ (optional)         */
 mlx_out_port(sccadr, 0x00);


				       /************************************/
				       /* WR 2 Interrupt Vector            */
				       /************************************/

 mlx_out_port(sccadr,  2);             /* Interrupt Vector = SCC_NUM       */
 mlx_out_port(sccadr, (channel / 2) << 6);


				       /************************************/
				       /* WR 3 Receive Parameters & Contr. */
				       /************************************/
 usart = 0;
 switch (databits)
   {                                   /* No. of databits = Bit 7 & 6      */
    case 5: break;                     /*               5 =     0   0      */
				       /*               6 =     1   0      */
    case 6: usart = usart | 0x80; break;
				       /*               7 =     0   1      */
    case 7: usart = usart | 0x40; break;
				       /*               8 =     1   1      */
    case 8: usart = usart | 0xC0; break;

    default: return(1);
   }
				       /* Bit 5=0                          */
				       /* Bit 4-2 ignored in async. mode   */
				       /* Bit 1=0 SYNC Character disabl.   */
				       /* Bit 0=0 Receiver disable         */

 mlx_out_port(sccadr,  3);             /* WR 3  Receive Parameters & Contr.*/
 mlx_out_port(sccadr, usart);

 rcvcnt = usart;                       /* store WR 3 settings in rcvcnt    */

				       /************************************/
				       /* WR 5 Transmit Parameters & Contr.*/
				       /************************************/


 usart = usart >> 1;                   /* Shift WR3 data one bit right     */

 mlx_out_port(sccadr,  5);             /* WR 5  Transm. Parameters & Contr.*/
 mlx_out_port(sccadr, usart);
				       /* store WR 5 settings for this ch. */
 tmtctr[micro_slot][channel] = usart;


				       /************************************/
				       /* WR 6 not used in asynchr. mode   */
				       /************************************/

				       /************************************/
				       /* WR 7 is only used by the ESCC    */
				       /************************************/

				       /************************************/
				       /* WR 8 is the TRANSMIT BUFFER      */
				       /************************************/

				       /************************************/
				       /* WR 9 Master Interrupt Control    */
				       /************************************/

 mlx_out_port(sccadr,  9);             /* WR 9 Master Interrupt Control    */
 mlx_out_port(sccadr, 0x00);




				       /************************************/
				       /* WR 10 Misc. Transmitter/Receiver */
				       /************************************/
 mlx_out_port(sccadr, 10);             /* WR 10 Misc. Transmitter/Receiver */
 mlx_out_port(sccadr, 0x00);


				       /************************************/
				       /* WR 11 Clock Mode Control         */
				       /************************************/

 mlx_out_port(sccadr, 11);             /* WR 11 Clock-Control              */

 if(use_brg == 0x00)
   mlx_out_port(sccadr, 0x05);         /* baudrate-generator not used      */
				       /* transmit and receive             */
				       /* must be clocked from RTxC-pin    */
 else
   mlx_out_port(sccadr, mode & 0x7E);  /* MSB and LSB must be 0            */

				       /************************************/
				       /* WR 12 Time Constant (Low-Byte)   */
				       /************************************/

 mlx_out_port(sccadr, 12);             /* Time-Constant (Low-Byte)         */
 mlx_out_port(sccadr, timec.lo_hi[0]);


				       /************************************/
				       /* WR 13 Time Constant (High-Byte)  */
				       /************************************/

 mlx_out_port(sccadr, 13);             /* Time-Constant (High-Byte)        */
 mlx_out_port(sccadr, timec.lo_hi[1]);


				       /************************************/
				       /* WR 14 Misc. Control              */
				       /************************************/

 mlx_out_port(sccadr, 14);             /* WR 14 Miscellaneous Control      */
 mlx_out_port(sccadr, 0x60);


 /**************************************************************************/
 /*      STAGE 2. Enables                                                  */
 /**************************************************************************/

				       /************************************/
				       /* WR 3 Receive Parameters & Contr. */
				       /************************************/

 mlx_out_port(sccadr,  3);             /* now Enable Receiver (Bit 0=1)    */
 mlx_out_port(sccadr, rcvcnt | 1);


				       /************************************/
				       /* WR 5 Transmit Parameters & Contr.*/
				       /************************************/

 mlx_out_port(sccadr,  5);             /* now Enable Transmitter (Bit 3=1) */
 tmtctr[micro_slot][channel] = tmtctr[micro_slot][channel] | 0x08;
 mlx_out_port(sccadr, tmtctr[micro_slot][channel] | 0x82);


				       /************************************/
				       /* WR 0 Command Register            */
				       /************************************/

				       /* WR 0 Init., Reset TxCRC          */
 mlx_out_port(sccadr, 0x80);


				       /************************************/
				       /* WR 14 Misc. Control              */
				       /************************************/

 mlx_out_port(sccadr, 14);             /* WR 14, Enable Baudrate-Generator */
 mlx_out_port(sccadr, 0x01);


				       /************************************/
				       /* WR 1 Interrupt Enable            */
				       /************************************/

 mlx_out_port(sccadr,  1);             /* Select if DMA (optional)         */
 mlx_out_port(sccadr, 0x00);


 /**************************************************************************/
 /*      STAGE 3. Interrupt Enables                                        */
 /**************************************************************************/

				       /************************************/
				       /* WR 15 Ext./Status Interrupt Con. */
				       /************************************/

 mlx_out_port(sccadr,  15);            /* Ext./Status Interrupt Enable     */
 mlx_out_port(sccadr, int_x_mode);


				       /************************************/
				       /* WR 0 Command Register            */
				       /************************************/

				       /* Reset Ext./Status (2x)           */
 mlx_out_port(sccadr, 0x10);

 mlx_out_port(sccadr, 0x10);


				       /************************************/
				       /* WR 1 Interrupt Enable            */
				       /************************************/

 mlx_out_port(sccadr,  1);             /* Enable receive, transmit and     */
				       /* external interrupt master        */
 mlx_out_port(sccadr, int_mode & 0x1F);


				       /************************************/
				       /* WR 9 Master Interrupt Control    */
				       /************************************/

 mlx_out_port(sccadr,  9);             /* WR 9,                            */
				       /* MIE = 1, Modify Vector V1-V3     */
 if(int_mode > 0) mlx_out_port(sccadr, 0x09);
				       /* MIE = 0 ..., no interrupt        */
 else             mlx_out_port(sccadr, 0x01);

 return(0);
}



/***************************************************************************/
/* Read Status of Transmit-Buffer                                          */
/*                                                                         */
/* This function reads the status of one channels transmit-buffer-empty    */
/* bit.                                                                    */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* Result = 1 : Transmit Buffer empty (TBE)                                */
/*                                                                         */
/* Result = 0 : last Character is still in Buffer                          */
/***************************************************************************/
byte EXPORT m048_scc_tbe(byte micro_slot, byte channel)
{
 byte tbe;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

				       /* Reset External Status Interrupt  */
 mlx_out_port(sccadr, 0x10);
 tbe = mlx_in_port(sccadr);

 return((tbe & 0x04) >> 2);
}



/***************************************************************************/
/* Read Status of Receive-Buffer                                           */
/*                                                                         */
/* This function reads the status of one channels receive-buffer-full bit. */
/* bit.                                                                    */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* Result = 1 : Receive Buffer full (RBF)                                  */
/*                                                                         */
/* Result = 0 : no Character received                                      */
/***************************************************************************/
byte EXPORT m048_scc_rbf(byte micro_slot, byte channel)
{
 byte rbf;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;
				       /* Reset External Status Interrupt  */
 mlx_out_port(sccadr, 0x10);
 rbf = mlx_in_port(sccadr);

 return(rbf & 0x01);
}



/***************************************************************************/
/* Read Status of CTS                                                      */
/*                                                                         */
/* This function reads the status of one channels cts-line.                */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* Result = 1 : CTS = 1                                                    */
/*                                                                         */
/* Result = 0 : CTS = 0                                                    */
/***************************************************************************/
byte EXPORT m048_scc_cts(byte micro_slot, byte channel)
{
 byte cts;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

 mlx_out_port(sccadr, 0x10);
 cts = mlx_in_port(sccadr);

 return((cts & 0x20) >> 5);
}



/***************************************************************************/
/* Transmit Character                                                      */
/*                                                                         */
/* The SCC must be initialized before !                                    */
/* Before transmitting the status of the trasnmit-buffer (TBE) is checked. */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* character  : Transmit Character                                         */
/*                                                                         */
/* Result     : 0=character sent, 1=transmit buffer not empty              */
/***************************************************************************/
byte EXPORT m048_transmit_scc_character(byte micro_slot, byte channel,
					byte character)
{
 byte tbe;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

 mlx_out_port(sccadr, 0x10);           /* Reset External Status Interrupt  */
 tbe = mlx_in_port(sccadr);            /* Transmit Buffer Empty ?          */

 if((tbe & 0x04) >> 2)                 /* No, Send 1 Byte                  */
   mlx_out_port(sccadr + 1, character);
 else
   return(1);                          /* Transmit Buffer is Full !        */

 return(0);
}



/***************************************************************************/
/* Receive Character                                                       */
/*                                                                         */
/* The SCC must be initialized before !                                    */
/* Before receiving the status of the receive-buffer (RBF) is checked.     */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* Result     : data = Received Character                                  */
/*                                                                         */
/* Return     : 0 OK, 1=RB nut full (nothing to read)                      */
/***************************************************************************/
byte EXPORT m048_receive_scc_character(byte micro_slot, byte channel,
				       byte *data)
{
 byte rbf;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

				       /* Reset External Status Interrupt  */
 mlx_out_port(sccadr, 0x10);
 rbf = mlx_in_port(sccadr) & 0x01;

				       /* Read Byte from Receive-Buffer    */
 if(rbf) *data = mlx_in_port(sccadr + 1);
 else return(1);

 return(0);
}



/***************************************************************************/
/* Set/Clear RTS                                                           */
/*                                                                         */
/* This procedure sets the rts-output of one sccs channel to the given     */
/* value in "rts" (0/1).                                                   */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* rts        : Value to set                                               */
/***************************************************************************/
void EXPORT m048_set_rts(byte micro_slot, byte channel, byte rts)
{
 byte usart;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;


 usart = tmtctr[micro_slot][channel];

 if(rts) usart = usart | 0x02;

 mlx_out_port(sccadr, 5);              /* WR 5                             */
 mlx_out_port(sccadr, usart);
}



/***************************************************************************/
/* Read Interrupt Vector, RR0, RR1 and RR3=Pending Register                */
/*                                                                         */
/* After this procedure, the interrupt service routine must clear the      */
/* pending interrupt for the interrupting channel.                         */
/*                                                                         */
/*  status in vector         : clear pending interrupt by                  */
/*                                                                         */
/*  1) Ext./Status Interrupt : Reset Ext./Status Interrupt in WR9          */
/*  2) Transmit Buffer Empty : Write next char. into transmit buffer WR8   */
/*  3) Receive Char. Avail.  : Read character out of receive buffer RR8    */
/*  4) Special Receive Cond. : Error Reset Command in WR0                  */
/*                                                                         */
/* At the end of the interrupt service routine, a reset highest IUS in WR0 */
/* must be issued.                                                         */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* Result     :                                                            */
/*                                                                         */
/* int_vector : Interrupt Vector including Status Bits (V1..V3)            */
/*                                                                         */
/* rr0        : Read Register 0, Transmit/Receive Ext./Status Register     */
/*                                                                         */
/* rr1        : Read Register 1, Special Receive Cond. Status              */
/*                                                                         */
/* rr3        : Read Register 3, Interrupt Pending Register                */
/***************************************************************************/
void EXPORT m048_scc_intak(byte micro_slot, byte *int_vector, byte *rr0, byte *rr1,
		    byte *rr3)
{
 unsigned char i;
 ushort sccadr;

 mlx_out_port(MBASIS + 0x18, 0x00);    /* set INTAK = low                  */

 for(i = 0; i < 48; i++)               /* dummy access to create delay     */
   mlx_in_port(MBASIS + 0x18);

 *int_vector = mlx_in_port(MBASIS);    /* read interrupt-vector            */

				       /* calc. scc_address of active SCC  */
 sccadr = MBASIS + (*int_vector >> 6) * 4;
 if(*int_vector & 0x08) sccadr += 2;   /* channel A !                      */

				       /* read RR0 (status)                */
 *rr0 = mlx_in_port(sccadr);
				       /* read RR1 (special rec. cond.)    */
 mlx_out_port(sccadr, 1);
 *rr1 = mlx_in_port(sccadr);
				       /* read RR3 (interrupt pending)     */
				       /* always read on channel A         */
 sccadr = MBASIS + (*int_vector >> 6) * 4 + 2;
 mlx_out_port(sccadr, 3);
 *rr3 = mlx_in_port(sccadr);

 return;
}



/***************************************************************************/
/* Get best values for to create a specific baudrate                       */
/*                                                                         */
/* This routines can be used to find the optimal parameters for FPGA and   */
/* SCC programming.                                                        */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* baudrate   : user baudrate (for example : 9600L, 19200L, ...)           */
/* quartz     : M-COM-8 quartz-oscillator-frequency (24000000L = 24MHz)    */
/*                                                                         */
/* Results    :                                                            */
/*                                                                         */
/* bauderror  : absolute error of baseclock (baseclk - desired clock)      */
/*                                                                         */
/* reg        : baseclk-register                                           */
/*                                                                         */
/* clk_mode   : SCC clock-mode (16.0, 32.0, 64.0)                          */
/*                                                                         */
/* timeconst  : SCC-timeconst counter value (for BRG, 16 Bit)              */
/*              if timeconst = -1 the BRG is not used                      */
/***************************************************************************/
void EXPORT m048_baudrate_calc(long baudrate, long quartz,
			       long *bauderror,
			       byte *basereg,
			       byte *clk_mode, ushort *timeconst,
			       byte *use_brg)
{
 byte clock_mode[3];                   /* SCC internal clock dividers      */
 long base;
 int help;
 byte x, y, i, postdiv;
 unsigned int j, maxtimeconst, maxclockmode;

 clock_mode[0] = 16;                   /* SCC internal clock divider       */
 clock_mode[1] = 32;
 clock_mode[2] = 64;


 maxclockmode = 3;                     /* clockmode /16 fits always        */

 maxtimeconst = 512;
				       /* limit timeconst                  */
 if(baudrate <   500L) maxtimeconst = 2048;
 if(baudrate > 38400L) maxtimeconst =   50;

 *bauderror = 200000000L;              /* set error max.                   */


 for(i = 0; i < maxclockmode; i++)     /* loop 1: clkmode of BRG (/16, ..) */
   {
				       /* 1. Check, if Baudrate with       */
				       /* Timeconst. possible              */
    for(j = maxtimeconst; j > 0L; j--) /* loop 2: timeconst of BRG         */
      {
       help = ((int)j + 2) * 2;
       base = l_multi((long)help, (long)clock_mode[i]);
       base = l_multi(base, baudrate);

       if(base > 8000000L) continue;

       if(base > 0L)
	 {                             /* compare values                   */
	  m048_check(quartz, base, clock_mode[i], j, 1,
		     &x, &y,
		     clk_mode, timeconst, use_brg, &postdiv, bauderror);
	 }

       if(*bauderror >= 0L && *bauderror <= 5L) break;
      } // j loop (timeconst of SCC BRG)


    if(*bauderror > 5L)                /* 2. Check, if Baudrate without    */
      {                                /* Timeconst. possible              */
       base = l_multi(baudrate, (long)clock_mode[i]);

       if(base > 8000000L) continue;

       if(base > 0L)
       m048_check(quartz, base, clock_mode[i], 0, 0, &x, &y, clk_mode,
		  timeconst, use_brg, &postdiv, bauderror);

       if(*bauderror >= 0L && *bauderror <= 5L) break;
      }
   } // i loop (clockmode of SCC BRG)


				       /* calculation finished :           */

				       /* create basereg variable          */
 *basereg  = (x | (y << 5)) & 0x3F;

 switch((int)postdiv)
   {
    case 5: *basereg |= 0x00; break;
    case 4: *basereg |= 0x40; break;
    case 3: *basereg |= 0x80; break;
    case 2: *basereg |= 0xC0; break;
   }
 return;
}




/***************************************************************************/
/* Sub-function of baudrate_calc (compares disired clock with fpga-clock)  */
/***************************************************************************/
void EXPORT m048_check(long quartz, long base, byte clk_mode,
		ushort timeconst, byte use_brg, byte *bestx, byte *besty,
		byte *bestclk_mode, ushort *besttimeconst,
		byte *bestuse_brg,
		byte *bestpostdiv, long *bestbauderror)
{
 long err;
 long fpgaclk, help;
 byte postdivide[4] = {5, 4, 3, 2};    /* FPGA internal clock divider      */
 byte up_factor, down_factor;
 byte x, y, p;

 for(p = 0; p < 4; p++)                /* loop 1: postdiv inside FPGA      */
   {                                   /* divides quartz /5, /4, /3 or /2  */
    help = l_divide((long)quartz, (long)postdivide[p]);

    for(y = 0; y < 2; y++)             /* loop 2: y bit 0/1                */
      {                                /* divides quartz with factor       */
       for(x = 0; x < 32; x++)         /* loop 3: x = 5 Bit Counter(0..31) */
	 {
				       /* baseclk factor inside FPGA       */
	  up_factor = ((y + 1) * x - y);
	  down_factor = ((y + 1) * x + 1);
	  if(x == 0)                   /* x = 0 && y = 0 : factor = 1      */
	    {                          /* x = 0 && y = 1 : not valid       */
	     up_factor = 1;
	     down_factor = 1;
	     if(y == 1) continue;
	    }

	  if(up_factor > 1)
	    {
	     fpgaclk = l_multi(help, (long)up_factor);
	     fpgaclk = l_divide(fpgaclk, (long)down_factor);
	    }
	  else fpgaclk = help;

	  if(fpgaclk > base) err = fpgaclk - base;
	  else err = base - fpgaclk;
				       /* abs. error                       */
	  if(err < *bestbauderror)
	    {
	     *bestbauderror  = err;
	     *bestclk_mode   = clk_mode;
	     *bestpostdiv    = postdivide[p];
	     *besttimeconst  = timeconst;
	     *bestx          = x;
	     *besty          = y;
	     *bestuse_brg   = use_brg;
	    }
	  if(err == 0L) break;

	 } // x loop (divider in fpga)
      } // y loop (divider in fpga)
   } // p loop (postdivider in fpga)
 return;
}


/***************************************************************************/
/* Set one channel (0..7) of M-COM-8 with specific baudrate.               */
/*                                                                         */
/* This routine calls m048_baudrate_calc... and checks if if the M-COM-8   */
/* has enough resources to generate the baudrate.                          */
/* This is done by evaluating the EEPROM data of the module.               */
/* If so, this function sets the parameters inside the SCC and the gate-   */
/* array (using m048_init_scc).                                            */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B, 2=channel C, 3 =channel D        */
/*              4=channel E, 5=channel F, 6=channel G, 7 =channel H        */
/*                                                                         */
/* baudrate   : user baudrate (for example : 9600L, 19200L, ...)           */
/*                                                                         */
/* flag       : selects how to create the settings                         */
/* flag     0 : returns error (1) if channel is used                       */
/*              returns error (2) if no baseclk available                  */
/*              checks if already used baseclks are possible               */
/*              checks if baseclks are available                           */
/* flag  1..4 : force setting to basereg no. (flag)                        */
/* flag     5 : returns error (2) if no baseclk available                  */
/*              checks if already used baseclks are possible               */
/*              checks if baseclks are available                           */
/* flag     6 : checks if already used baseclks are possible               */
/*              checks if baseclks are available                           */
/*              if all baseclks are used, conncect channel to best baseclk */                                                                         
/*                                                                         */
/* Parameters (as used in m048_init_scc())                                 */
/*                                                                         */
/* mode       : Select Clock-Mode Control (=WR11)                          */
/*                                                                         */
/* int_mode   : =Write Register 1 of a SCCs channel (WR 1, lower 5 bits)   */
/*                                                                         */
/* int_x_mode : =Write Register 15 of a SCCs channel (WR 15)               */
/*                                                                         */
/* databits   : 5, 6, 7 or 8 Databits (transmit and receive)               */
/*                                                                         */
/* stopbits   : 0=no stopbits, 1=1 stopbit, 2=1,5 stopbits, 3=2 stopbits   */
/*                                                                         */
/* parity_type: 0=no parity , 1=odd parity, 2=even parity type             */
/*                                                                         */
/* Results    :                                                            */
/*                                                                         */
/* bauderror  : absolute error of baseclock (baseclk - desired clock)      */
/*                                                                         */
/* Return     : status: 0=ok                                               */
/*                      1=channel already used (flag = 0)                  */
/*                      2=no baseclk register available (flag = 5)         */
/***************************************************************************/
byte EXPORT m048_set_channel(byte micro_slot, byte channel, long baudrate,
			     byte flag,
			     byte scc_mode, byte scc_int_mode,
			     byte scc_int_x_mode, byte scc_databits,
			     byte scc_stopbits, byte scc_parity_type,
			     long *bauderror, byte *used_clkmode)
{
 byte clock_mode[3];
 ushort timeconst, besttimeconst, timec;
 byte   basisreg, x, y, postdiv, use_brg;
 int    help;
 long   baud_error, best_error;
 long   fpgaclk, base, lhelp;
 byte   clkmode, i, j;
 byte   found = 0;                     /* flag if settings successful      */
 byte   avail = 0;                     /* number of available baseclk-reg  */



 clock_mode[0] = 16;                   /* SCC internal clock divider       */
 clock_mode[1] = 32;
 clock_mode[2] = 64;

 *used_clkmode = 0;

				       /* FLAG 0                           */
				       /* check if channel is used         */
 if((flag == 0) && (m048clksel[micro_slot][channel] != 0)) return(1);


				       /* calc settings for this baudrate  */
 m048_baudrate_calc(baudrate, l_multi(m048quartz[micro_slot], 1000L), &baud_error, &basisreg,
		    &clkmode, &timeconst, &use_brg);

 *used_clkmode = clkmode;

 *bauderror = baud_error;
 best_error = baud_error;


				       /* FLAG 1...4                       */
 if((flag > 0) && (flag < 5))          /* force use of baseclk register    */
   {
    m048_init_scc(micro_slot, channel,
		  scc_mode, scc_int_mode, scc_int_x_mode,
		  timeconst, use_brg, clkmode, scc_databits, scc_stopbits,
		  scc_parity_type);

    m048baseclk[micro_slot][flag - 1] = basisreg;
				       /* set channel-select register      */
    mlx_out_port(MBASIS + M048_CHANNELSEL, (byte)channel);
				       /* set clock-select register        */
    mlx_out_port(MBASIS + M048_CLKSEL, (byte)(flag));
				       /* set avail.-baseclk register      */
    mlx_out_port(MBASIS + M048_BASECLK1 + flag - 1, (byte)basisreg);

    return(0);
   }



				       /* FLAG 0, 5 & 6:                   */

				       /* check, if basisreg already used  */
 for(i = 0; i < 4; i++)
   {
    if(basisreg == m048baseclk[micro_slot][i])
      {                                /* ok, use this baseclk too         */
       m048_init_scc(micro_slot, channel,
		     scc_mode, scc_int_mode, scc_int_x_mode,
		     timeconst, use_brg, clkmode, scc_databits, scc_stopbits,
		     scc_parity_type);

				       /* set channel-select register      */
       mlx_out_port(MBASIS + M048_CHANNELSEL, (byte)channel);
				       /* set clock-select register        */
       mlx_out_port(MBASIS + M048_CLKSEL, (byte)(i + 1));
       m048clksel[micro_slot][channel] = (byte)(i + 1);
       found = 1;
       return(0);                      /* successful                       */
      }
   }



 if(!found)                            /* try other baseclks               */
   {                                   /* with different scc settings      */
    for(i = 0; i < 4; i++)
      {                                /* calc clock of this baseclk       */
       for(j = 0; j < 3; j++)
	 {
	  x = m048baseclk[micro_slot][i] & 0x1F;
	  y = (m048baseclk[micro_slot][i] & 0x20) >> 5;
	  postdiv  = 5 - ((m048baseclk[micro_slot][i] & 0xC0) >> 6);

	  fpgaclk  = l_multi((long)m048quartz[micro_slot], 1000L);
	  fpgaclk  = l_divide(fpgaclk, (long)postdiv);
	  help     =  (int)x * (int)(y + 1) + 1;
	  fpgaclk  = l_divide(fpgaclk, (long)help);
	  help     = (int)x * (int)(y + 1) - (int)y;
	  fpgaclk  = l_multi((long)help, fpgaclk);

	  lhelp    = l_multi((long)(clock_mode[j] * 2), baudrate) - 2L;
	  timec    = (ushort)l_divide(fpgaclk, lhelp) - 1;
	  base     = l_multi((long)timec + 2L, (long)(clock_mode[j] * 2));
	  base     = l_multi(baudrate, base);

				       /* abs. error                       */
	  if(fpgaclk > base) baud_error = (long)(fpgaclk - base);
	  else baud_error = (long)(base - fpgaclk);
	  if(baud_error <= best_error)
	    {
	     best_error = baud_error;
	     found = i + 1;
	     besttimeconst = timec;
	     *used_clkmode = clock_mode[j];
	     break;
	    }
	 }
      }

    if(best_error <= baud_error)       /* set scc with baseclk(found)      */
      {
       *bauderror = best_error;
       m048_init_scc(micro_slot, channel,
		     scc_mode, scc_int_mode, scc_int_x_mode,
		     besttimeconst, 1, *used_clkmode, scc_databits, scc_stopbits,
		     scc_parity_type);
				       /* set channel-select register      */
       mlx_out_port(MBASIS + M048_CHANNELSEL, (byte)channel);
				       /* set clock-select register        */
       mlx_out_port(MBASIS + M048_CLKSEL, (byte)found);
       m048clksel[micro_slot][channel] = (byte)(found);
      }
   }

 if(found > 0) return(0);              /* successful connected to an       */
				       /* already used baseclk             */



				       /* now try to use a new baseclk:    */

 if(!found)                            /* check if another baseclk is free */
   {
    for(j = 0; j < 8; j++)
      if((m048clksel[micro_slot][j] > 0) && (m048clksel[micro_slot][j] < 5))
	avail |= 1 << (m048clksel[micro_slot][j] - 1);

				       /* no more baseregs available       */
    if((avail == 0x0F) && (flag == 5)) return(2);


    for(i = 0; i < 4; i++)
      {
       if(!((avail >> i) & 0x01))      /* this one is free                 */
	 {
	  m048_init_scc(micro_slot, channel,
			scc_mode, scc_int_mode, scc_int_x_mode,
			timeconst, use_brg, clkmode, scc_databits, scc_stopbits,
			scc_parity_type);
	  m048baseclk[micro_slot][i] = basisreg;
				       /* set channel-select register      */
	  mlx_out_port(MBASIS + M048_CHANNELSEL, (byte)channel);
				       /* set clock-select register        */
	  mlx_out_port(MBASIS + M048_CLKSEL, (byte)(i + 1));
	  m048clksel[micro_slot][channel] = (byte)(i + 1);
				       /* set avail.-baseclk register      */
	  mlx_out_port(MBASIS + M048_BASECLK1 + i, (byte)basisreg);
	  found = 1;
	  break;
	 }
      }
   }


				       /* FLAG = 6                         */

 if(!found)                            /* take best available baseclk      */
   {                                   /* with different scc settings      */
    *bauderror   = 20000000L;
    for(i = 0; i < 4; i++)
      {                                /* calc clock of this baseclk       */
       for(j = 0; j < 3; j++)
	 {
	  x = m048baseclk[micro_slot][i] & 0x1F;
	  y = (m048baseclk[micro_slot][i] & 0x20) >> 5;
	  postdiv  = 5 - ((m048baseclk[micro_slot][i] & 0xC0) >> 6);

	  fpgaclk  = l_multi((long)m048quartz[micro_slot], 1000L);
	  fpgaclk  = l_divide(fpgaclk, (long)postdiv);
	  help     =  (int)x * (int)(y + 1) + 1;
	  fpgaclk  = l_divide(fpgaclk, (long)help);
	  help     = (int)x * (int)(y + 1) - (int)y;
	  fpgaclk  = l_multi((long)help, fpgaclk);

	  lhelp    = l_multi((long)(clock_mode[j] * 2), baudrate) - 2L;
	  timec    = (ushort)l_divide(fpgaclk, lhelp) - 1;
	  base     = l_multi((long)timec + 2L, (long)(clock_mode[j] * 2));
	  base     = l_multi(baudrate, base);

				       /* abs. error                       */
	  if(fpgaclk > base) baud_error = (long)(fpgaclk - base);
	  else baud_error = (long)(base - fpgaclk);
	  if(baud_error <= *bauderror)
	    {
	     *bauderror = baud_error;
	     found = i + 1;
	     besttimeconst = timec;
	     *used_clkmode = clock_mode[j];
	     break;
	    }
	 }
      }

   if(found)
      {
       *bauderror = best_error;
       m048_init_scc(micro_slot, channel,
		     scc_mode, scc_int_mode, scc_int_x_mode,
		     besttimeconst, 1, *used_clkmode, scc_databits, scc_stopbits,
		     scc_parity_type);
				       /* set channel-select register      */
       mlx_out_port(MBASIS + M048_CHANNELSEL, (byte)channel);
				       /* set clock-select register        */
       mlx_out_port(MBASIS + M048_CLKSEL, (byte)found);
       m048clksel[micro_slot][channel] = (byte)(found);
      }
   }

 return(0);
}



static long EXPORT l_multi(long x, long y)
{
 long y1, y2, y3;

				       /* lowword(x) * lowword(y)          */
 asm mov ax, word ptr x;               /* lowword(x) in ax                 */
 asm mul word ptr y;                   /* lowword(y) * ax                  */
 asm mov word ptr y1, ax;              /* lowword(res.)=ax in lowword(y1)  */
 asm mov word ptr y1 + 2, dx;          /* highword(res.)=dx in highword(y1)*/

				       /* lowword(x) * highword(y)         */
 asm mov ax, word ptr x;               /* lowword(x) in ax                 */
 asm mul word ptr y + 2                /* highword(y) * ax                 */
 asm mov word ptr y2, 0;               /* lowword(y2) = 0 (= 16 Bit SHL)   */
 asm mov word ptr y2 + 2, ax;          /* lowword(res.) in highword(y2)    */
				       /* dx is overflow, result <= 32 Bit */

				       /* highword(x) * lowword(y)         */
 asm mov ax, word ptr x + 2;           /* highword(x) in ax                */
 asm mul word ptr y;                   /* lowword(y) * ax                  */
 asm mov word ptr y3, 0;               /* lowword(y3) = 0 (= 16 Bit SHL)   */
 asm mov word ptr y3 + 2, ax;          /* lowwrod(res.) in highword(y3)    */
				       /* dx is overflow, result <= 32 Bit */

				       /* highword(x) * highword(y)        */
				       /* not used (result = 32 Bit)       */

				       /* add all results (y1 + y2 + y3)   */
 asm mov ax,word ptr y2;               /* lowword(y2) + lowword(y1)        */
 asm add word ptr y1, ax;              /* result in lowword(y1)            */
 asm mov ax,word ptr y2 + 2;           /* highword(y2) + highword(y1) + C  */
 asm adc word ptr y1 + 2, ax;          /* result in highword(y1)           */

 asm mov ax,word ptr y3;               /* lowword(y3) + lowword(y1)        */
 asm add word ptr y1, ax;              /* result in lowword(y1)            */
 asm mov ax,word ptr y3 + 2;           /* highword(y3) + highword(y1) + C  */
 asm adc word ptr y1 + 2, ax;          /* result in highword(y1)           */

 return(y1);
}



static long EXPORT l_divide(long x, long y)
{
 long result;

 asm cmp word ptr y + 2, 0;            /* count if highword(y) > 0         */
 asm jnz count;
				       /* else divide                      */

 asm mov ax, word ptr x + 2;           /* 1. calc highword(result)         */
 asm xor dx, dx;
 asm div word ptr y;                   /* ax = highword(x) / lowword(y)    */
				       /* dx = rest                        */

 asm mov word ptr result + 2, ax;      /* save highword of result          */

 asm mov ax, word ptr x;               /* calc lowword of result           */
 asm div word ptr y;

 asm mov word ptr result, ax;          /* save lowword of result           */
 asm jmp ende;

 count:
 asm mov word ptr result, 0;
 asm mov word ptr result + 2, 0;

 test:
 asm mov ax, word ptr y;
 asm sub word ptr x, ax                /* lowword(x) -= lowword(y)         */
 asm mov ax, word ptr y + 2;
 asm sbb word ptr x + 2, ax;           /* highword(x) -= (highword(y) + c) */
 asm jc  ende;
 asm add word ptr result, 1;           /* result++                         */
 asm adc word ptr result + 2, 0;
 asm jnc test;

 ende:
 return(result);

}
