#ifndef _m048_lib_
#define _m048_lib_
/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 library for SPB module M-COM-8                 **/
/**                                                                       **/
/**                             Version 1.B.                              **/
/**                                                                       **/
/**  This file contains routines for Borland C++/3.1 to allow the user,   **/
/**  to write programs for the MODULAR-4 in "C"                           **/
/**                                                                       **/
/**             Project Start    : 01.07.1995                             **/
/**             Last change      : 11.12.1995                             **/
/**                                                                       **/
/**                         Author : A.H.                                 **/
/**                         Status : o.k.                                 **/
/**                                                                       **/
/**                                                                       **/
/** Special comments:                                                     **/
/**                                                                       **/
/** This library was tested with Borland C++ Version 3.1.                 **/
/**                                                                       **/
/** No parameters are checked.                                            **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1995                           **/
/**                                                                       **/
/***************************************************************************/

/* The switches for the library are set in the configuration files of Pascal
   (*.TP file) und C (project-file). They can also be set in this source file:

   Then the following #defines have to exist:

   If you use Pascal, #define BORLAND_PASCAL has to be included (default is C).

   If the library shall be used as realtime library on the MODULAR-4/486
   #define ML8 and #define ML8RT have to be included (default is PC library).

   If you work with Windows, this is detected by the compiler. No #defines have
   to be included.
*/

#ifndef byte
	#define byte        unsigned char
#endif

#ifndef ulong
	#define ulong       unsigned long
#endif

#ifndef ushort
	#define ushort      unsigned short
#endif

#define lobyte(w)           ((byte)(w))
#define hibyte(w)           ((byte)((ushort)(w) >> 8))

				       /* Modul-Basisadresse                             */
#define MBASIS  micro_slot_basis[micro_slot]

#define M048_CHANNELSEL          0x10  /* Channel Select                   */
#define M048_CLKSEL              0x11  /* Clock Select Register            */
#define M048_BASECLK1            0x12  /* Base-Clock 1 Register            */
#define M048_BASECLK2            0x13  /* Base-Clock 1 Register            */
#define M048_BASECLK3            0x14  /* Base-Clock 1 Register            */
#define M048_BASECLK4            0x15  /* Base-Clock 1 Register            */
#define M048_INTSEL              0x16  /* Interrupt Select                 */
#define M048_PCLKSEL             0x17  /* PCLK Select                      */
#define M048_READ_FPGA_VERSION   0x18  /* Read FPGA-Version (8Bit)         */


#ifdef __DLL__
  #define EXPORT        _export far pascal
  #define PFAR          far pascal
#elif UNIX
  #define EXPORT
  #define PFAR
#else
  #define EXPORT        far pascal
  #define PFAR          far pascal
#endif


#if defined(__cplusplus)
extern "C" {
#endif



/* Init module library                            */
void EXPORT m048_bib_startup(byte micro_slot);

/* Reset SCC                                      */
void EXPORT m048_scc_reset(byte micro_slot, byte scc_num);

/* Initialize SCC-Channel                         */
byte EXPORT m048_init_scc(byte micro_slot, byte channel,
			  byte mode, byte int_mode, byte int_x_mode,
			  ushort timeconst, byte use_brg, byte clkmode,
			  byte databits, byte stopbits, byte parity_type);

/* Read SCC-TBE Bit                               */
byte EXPORT m048_scc_tbe(byte micro_slot, byte channel);

/* Read SCC-RBF Bit                               */
byte EXPORT m048_scc_rbf(byte micro_slot, byte channel);

/* Read SCC-CTS Bit                               */
byte EXPORT m048_scc_cts(byte micro_slot, byte channel);

/* Send Character                                 */
byte EXPORT m048_transmit_scc_character(byte micro_slot, byte channel,
					byte character);

/* Receive Character                              */
byte EXPORT m048_receive_scc_character(byte micro_slot, byte channel,
				       byte *data);

/* Set/Clear SCC-RTS                              */
void EXPORT m048_set_rts(byte micro_slot, byte channel, byte rts);


/* get interrupt vector, rr0, rr1 & rr3 (pending) */
void EXPORT m048_scc_intak(byte micro_slot, byte *int_vector, byte *rr0, byte *rr1,
			   byte *rr3);

/* calculate baudrate adjustments                 */
void EXPORT m048_baudrate_calc(long baudrate, long quartz,
			       long *bauderror, byte *reg,
			       byte *clk_mode, ushort *timeconst, byte *use_brg);
/* sub function of baudrate_calc                  */
void EXPORT m048_check(long quartz, long base, byte clk_mode,
		       ushort timeconst, byte use_brg, byte *bestx, byte *besty,
		       byte *bestclk_mode, ushort *besttimeconst, byte *bestuse_brg,
		       byte *bestpostdiv, long *bestbauderror);

/* set channel for baudrate (incl. fpga)          */
byte EXPORT m048_set_channel(byte micro_slot, byte channel, long baudrate,
			     byte flag, byte scc_mode, byte scc_int_mode,
			     byte scc_int_x_mode,
			     byte scc_databits, byte scc_stopbits,
			     byte scc_parity_type, long *bauderror,
			     byte *usedclkmode);

/* signed long multiplication (for compatibility) */
static long EXPORT l_multi(long x, long y);

/* signed long division       (for compatibility) */
static long EXPORT l_divide(long x, long y);


#if defined(__cplusplus)
} /*  End of block extern "C" */
#endif

#endif
