/***************************************************************************/
/**            M-COM-8 string transmit (NI-Task, MODULAR-4/486)           **/
/**                                                                       **/
/**  This program (NI-Task) transmits a string using module M-COM-8.      **/
/**  It uses the module-library M048_LIB.                                 **/
/**  Please note, that the EEPROM-contents are used to program the module.**/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**  Installation Reference:                                              **/
/**                                                                       **/
/**  program-number   : 330h                                              **/
/**  task-number      : defined by user                                   **/
/**  interrupt-source : 1 (not used, NI-Task)                             **/
/**  data-range       : 0                                                 **/
/**  flags            : 0980h (don't activate, call Auto-init, NI-Task)   **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                     for Borland C++ (Version 3.1)                     **/
/**                                                                       **/
/**                             Version 1.A.                              **/
/**                                                                       **/
/**                               date : 07/26/95                         **/
/**                        last change : --.--.--                         **/
/**                             status : not released                     **/
/**                             author : A.H.                             **/
/**                                                                       **/
/**                (c) 1995 SORCUS Computer GmbH Heidelberg               **/
/**                                                                       **/
/***************************************************************************/

#include "stdlib.h"
#include "dos.h"
#include "string.h"
				       /* header of real-time library      */
#include "c:\sorcus\ml8\rt\bc\ml8rtbib.h"
#include "..\..\..\..\c\m048_lib.h"    /* header of module-library         */

				       /* general defines                  */
#define PRG_NUMBER     0x330           /* program-number = 330h            */
#define VERSION        '1'             /* program-version                  */
#define REVISION       'A'             /* program-revision                 */

				       /* status-definitions of task       */
#define READY          0               /* program ready                    */
#define RUNNING        2               /* programm active                  */
#define ERROR          3               /* error                            */
#define STOPPED        4               /* program halted                   */


#define MODUL          3               /* 1..4..10                         */

#define SCC_CHANNEL    7               /* 0 = A , 1 = B, ..., 7 = H        */
#define SCC_MODE       0x56            /* clock mode = master              */
#define SCC_INTMODE    0x00            /* no interrupts                    */
#define SCC_INT_X_MODE 0x00            /* no ext. interrupts               */
#define SCC_QUARTZ     25000000L       /* frequence of clk-oszillator      */
#define SCC_BAUDRATE   19200L          /* desired baudrate                 */
#define SCC_DATABITS   8               /* number of databits (5..8)        */
#define SCC_STOPBITS   1               /* 0=0 stopbits, 1=1, 2=1,5, 3=2    */
#define SCC_PARTYPE    0               /* 0=no parity, 1=odd, 2=even       */

 const unsigned short channels[8] = {0x02, 0x00,
				     0x06, 0x04,
				     0x0A, 0x08,
				     0x0E, 0x0C};

#define CHANNEL channels[SCC_CHANNEL]  /* Channels A..H                    */



unsigned short micro_slot_basis[11] = {0, 0x400, 0x500,
				       0x600, 0x700,
				       0x420, 0x440,
				       0x460, 0x480,
				       0x4A0, 0x4C0};


#define MBA micro_slot_basis[MODUL]

				       /* Program-Descriptor-Table (PDT)   */
struct pdt_type
{
 unsigned char type;                   /* type of PDT                      */
 unsigned char lengt;                  /* length of PDT-header             */
 unsigned int  proc_num;               /* number of procedures             */
 unsigned int  prg_number;             /* program-number                   */
 unsigned char version;                /* program-version                  */
 unsigned char revision;               /* program revision                 */
 unsigned char cpu_type;               /* processor-type                   */
 unsigned char coproz_type;            /* co-prozessor-type                */
 unsigned char language;               /* programming language             */
 unsigned char prg_type;               /* program-type                     */

 unsigned int  flags;                  /* installation-flags               */
 unsigned int  interrupt_num;          /* number of the used interrupt     */

 unsigned long data_adr;               /* start-address of data-range      */
 unsigned long data_size;              /* size of data-range               */
 unsigned long data_min;               /* minimum size of data-range       */
 unsigned long data_max;               /* maximum size of data-range       */

 unsigned long parameter_adr;          /* address of parameter-range       */
 unsigned int  parameter_size;         /* number of parameters             */

 unsigned long hyper_adr;              /* start-address of hypertext-range */
 unsigned int  res1;
 unsigned long res2;

 void          (*main_proc)();         /* address of Main-procedure        */
 void          (*auto_init)();         /* address of Auto-Init procedure   */
 void          (*start_proc)();        /* address of Start-procedure       */
 void          (*stop_proc)();         /* address of Stop-procedure        */
}pdt;

				       /* definition of parameter-range    */
struct parameter_type
{
 tdt_type       tdt;                   /* Task-Descriptor-Table TDT        */
 unsigned char  status;                /* actual status of program         */
}parameter;


void auto_init(void);                  /* prototypes of procedures         */
void main_task(void);
void start(void);
void stop(void);
void init_pdt(void);
void init_module(void);


char string[] = "Transmitting Data using M-COM-8 !\n\r";



void auto_init(void)                   /* AUTO-INIT                        */
{
 ml8rt_entry();                        /* save registers and prepare       */

 ml8rt_local_led_off();                /* on-board-LED off                 */

 ml8rt_exit();                         /* restore registers                */
}


void start(void)                       /* START-PROCEDURE                  */
{
 ml8rt_entry();                        /* save registers and prepare       */

 ml8rt_local_led_off();

 init_module();

 parameter.status = RUNNING;           /* set status "running"             */

 ml8rt_wakeup_task(parameter.tdt.task);/* enable task                      */

 ml8rt_exit();                         /* restore registers                */
}


void stop(void)                        /* STOP-PROCEDURE                   */
{
 ml8rt_entry();                        /* save registers and prepare       */

 ml8rt_sleep_task(parameter.tdt.task); /* disable task                     */
 parameter.status = STOPPED;           /* set status "stopped"             */

 ml8rt_exit();
}


void main_task(void)                   /* MAIN-PROCEDURE OF THE TASK       */
{
static unsigned int i = 0;

 ml8rt_entry();                        /* save registers and prepare       */

 ml8rt_local_led_on();

 if(i < strlen(string))
   {                                   /* try to transmit character        */
    if(m048_transmit_scc_character(MODUL, SCC_CHANNEL, string[i]) == 0) i++;                      /* if successful, increment pointer */
   }

 else ml8rt_local_led_off();           /* string transmitted               */

 ml8rt_exit();                         /* restore registers                */
}


void init_pdt(void)                    /* PDT-INIT                         */
{
 pdt.type        = 1;                  /* set PDT-type                     */
 pdt.lengt       = _PDT_LENGTH;        /* length of PDT-header             */
 pdt.proc_num    = 4;                  /* number of procedures             */
 pdt.prg_number  = PRG_NUMBER;         /* program-number                   */
 pdt.version     = VERSION;            /* program-version                  */
 pdt.revision    = REVISION;           /* programm-revision                */
 pdt.cpu_type    = _486DX;             /* CPU-type                         */
 pdt.coproz_type = _487DX;             /* Co-Prozessor-type                */
 pdt.language    = _CPP31;             /* program-language = C++           */
 pdt.prg_type    = _USER_PRG;          /* program-type: user-program       */
 pdt.flags       = 0;                  /* flags                            */
 pdt.flags       = _NI_TASK +
		   _PDT_INFO_ENABLE +
		   _LOCAL_DATA +
		   _LOCAL_PARAMETER +
		   _FIXED_DATASIZE;
 pdt.interrupt_num = 0;                /* interrupt-number                 */
 pdt.data_adr    = 0;                  /* address of data-range            */
 pdt.data_size   = 0;                  /* size of data-range               */
 pdt.data_min    = 0;                  /* min. size of data-range          */
 pdt.data_max    = 0;                  /* max. size of data-range          */
				       /* start address of parameter-range */
 ml8rt_phys_adr(&parameter, &pdt.parameter_adr);
 pdt.parameter_adr  = pdt.parameter_adr + sizeof(tdt_type);
				       /* number of parameter              */
 pdt.parameter_size = sizeof(parameter) - sizeof(tdt_type);
 pdt.hyper_adr  = 0;                   /* address of hypertext-range       */
 pdt.main_proc  = main_task;           /* address of main procedure        */
 pdt.auto_init  = auto_init;           /* address of auto_init procedure   */
 pdt.start_proc = start;               /* address of start procedure       */
 pdt.stop_proc  = stop;                /* address of stop procedure        */
}



void main (void)                       /* PREPARE                          */
{
 init_pdt();                           /* PDT init.                        */
 ml8rt_set_pdt_adr(&pdt);              /* return address of PDT to OS      */
}


void init_module(void)
{
 unsigned short timeconst;
 unsigned char basis_reg, clkmode, use_brg;
 long baud_error;

 m048_bib_startup(MODUL);              /* start library                    */

				       /* reset scc                        */
 m048_scc_reset(MODUL, (unsigned char)(SCC_CHANNEL / 2));

				       /* calculate baudrate-parameters    */
 m048_baudrate_calc(SCC_BAUDRATE, SCC_QUARTZ, &baud_error,
		    &basis_reg,
		    &clkmode, &timeconst, &use_brg);

 outportb(MBA + 0x12, basis_reg);      /* set baseclk1-register            */
				       /* set channel/clksel-register      */
 outportb(MBA + 0x10, SCC_CHANNEL);
 outportb(MBA + 0x11, 0x01);           /* set to baseclk1                  */

				       /* SCC-init.                        */
 m048_init_scc(MODUL, SCC_CHANNEL, SCC_MODE, SCC_INTMODE, SCC_INT_X_MODE,
	       timeconst, use_brg, clkmode, SCC_DATABITS, SCC_STOPBITS, SCC_PARTYPE);

 return;
}
