/***************************************************************************/
/**                 M-COM-8 Test Interrupts (MODULAR-4/486)               **/
/**                                                                       **/
/**  This program shows the interrupt handling for the module M-COM-8.    **/
/**  It uses the module-library M048_LIB to initialize one SCC.           **/
/**  A change on the CTS line sets the SCC-INT pin low, the DI-Task       **/
/**  works as the interrupt service routine, reading the irq-vector       **/
/**  and the interrupt pending register (generating the INTAK-cycle).	  **/
/**  Please note, that this program uses IRQ-C. You must set the          **/
/**  interrupt-register of the module M-COM-8 to 4 (IRQ-C) e.g. write     **/
/**  '0004' into WORD-5 of the module-EEPROM.                             **/
/**                                                                       **/
/**                                                                       **/
/**  Installation Reference :                                             **/
/**                                                                       **/
/**  program-number   : 331h						  **/
/**  task-number      : beliebig                                          **/
/**  interrupt-source : 7dh (IRQ-C, Master-Controller)                    **/
/**  data-range       : 0                                                 **/
/**  flags            : 0982h (don't activate task, call auto-init)       **/
/**                                                                       **/
/**                                                                       **/
/**                         Author : A.H.                                 **/
/**                         Status : ----                		  **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                       Borland C++ (Version 3.1)                       **/
/**                                                                       **/
/**                             Version 1.C.                              **/
/**                                                                       **/
/**          (c) 1992 - 1995 SORCUS Computer GmbH Heidelberg              **/
/***************************************************************************/

#include "stdlib.h"
#include "dos.h"

				       /* header of real-time library      */
#include "c:\sorcus\ml8\rt\bc\ml8rtbib.h"
#include "..\..\..\..\c\m048_lib.h"    /* header of module-library         */

				       /* general defines                  */
#define PRG_NUMBER 0x331               /* program-number = 331h            */
#define VERSION    '1'                 /* program-version                  */
#define REVISION   'C'                 /* program-revision                 */

				       /* status-definitions of task       */
#define READY      0                   /* program ready                    */
#define RUNNING    2                   /* programm active                  */
#define ERROR      3                   /* error                            */
#define STOPPED    4                   /* program halted                   */

#define MODUL          3               /* 1..4..10                         */

#define SCC_NUM        3               /* 0..3        			   */
#define SCC_CHANNEL    7               /* 0 = A , 1 = B, ... 7 = H         */
#define SCC_MODE       0x56            /* Master-mode                      */
#define SCC_INTMODE    0x01            /* WR 1, Ext./Status Interrupt Enab.*/
#define SCC_INT_X_MODE 0x20            /* WR 15, Ext.Int.Cond.: CTS change */
#define SCC_TIMECONST  24              /* Timeconst for BRG                */
#define SCC_CLKMODE    16              /* :1, :16, :32, :64                */
#define SCC_DATABITS   8               /* number of databits (5..8)        */
#define SCC_STOPBITS   1               /* 0=0 stopbits, 1=1, 2=1,5, 3=2    */
#define SCC_PARTYPE    0               /* 0=no parity, 1=odd, 2=even       */
#define SCC_USE_BRG    1               /* 0=no brg, 1=use brg              */


const unsigned short channels[8] = {0x02, 0x00,
				    0x06, 0x04,
				    0x0A, 0x08,
				    0x0E, 0x0C};



const unsigned short mba[11] = {0, 0x400, 0x500,
				   0x600, 0x700,
			           0x420, 0x440,
				   0x460, 0x480,
				   0x4A0, 0x4C0};


				       /* Program-Descriptor-Table (PDT)   */
struct pdt_type
{
 unsigned char type;                   /* type of PDT                      */
 unsigned char lengt;                  /* length of PDT-header 	           */
 unsigned int  proc_num;               /* number of procedures             */
 unsigned int  prg_number;             /* program-number                   */
 unsigned char version;                /* program-version                  */
 unsigned char revision;               /* program revision                 */
 unsigned char cpu_type;               /* processor-type                   */
 unsigned char coproz_type;            /* co-prozessor-type                */
 unsigned char language;               /* programming language             */
 unsigned char prg_type;               /* program-type                     */

 unsigned int  flags;                  /* installation-flags               */
 unsigned int  interrupt_num;          /* number of the used interrupt     */

 unsigned long data_adr;               /* start-address of data-range      */
 unsigned long data_size;              /* size of data-range               */
 unsigned long data_min;               /* minimum size of data-range       */
 unsigned long data_max;               /* maximum size of data-range       */

 unsigned long parameter_adr;          /* address of parameter-range       */
 unsigned int  parameter_size;         /* number of parameters             */

 unsigned long hyper_adr;              /* start-address of hypertext-range */
 unsigned int  res1;
 unsigned long res2;

 void          (*main_proc)();         /* address of Main-procedure        */
 void          (*auto_init)();         /* address of Auto-Init procedure   */
 void          (*start_proc)();        /* address of Start-procedure       */
 void          (*stop_proc)();         /* address of Stop-procedure        */
}pdt;

				       /* definition of parameter-range    */
struct parameter_type
{
 tdt_type       tdt;                   /* Task-Descriptor-Table TDT        */
 unsigned char  modul;                 /* modul-slot                       */
 unsigned char  scc_channel;           /* channel to initialize            */
 unsigned char  status;                /* actual status of program         */
 unsigned char  int_vector;
 unsigned char  rr0;
 unsigned char  rr1;
 unsigned char  rr3;
}parameter;


void auto_init(void);                  /* prototypes of procedures         */
void main_task(void);
void start(void);
void stop(void);
void init_pdt(void);

void auto_init(void)                   /* AUTO-INIT                        */
{
 ml8rt_entry();                        /* save registers and prepare       */
 ml8rt_external_led_off();             /* external LED off                 */
 parameter.scc_channel = SCC_CHANNEL;
 parameter.modul       = MODUL;
 ml8rt_local_led_off();                /* on-board-LED off                 */
 ml8rt_exit();                         /* restore registers                */
}


void start(void)                       /* START-PROCEDURE                  */
{
 ml8rt_entry();

 ml8rt_local_led_off();

 m048_bib_startup(parameter.modul);
 m048_scc_reset(parameter.modul, (parameter.scc_channel / 2));

				       /* SCC-init.                        */
 m048_init_scc(parameter.modul, parameter.scc_channel, SCC_MODE,
	       SCC_INTMODE, SCC_INT_X_MODE,
	       SCC_TIMECONST, SCC_USE_BRG, SCC_CLKMODE, SCC_DATABITS,
	       SCC_STOPBITS, SCC_PARTYPE);

				       /* set IRQ-C in FPGA of module      */
 outportb(mba[parameter.modul] + 0x16, 4);

 parameter.status = RUNNING;           /* set status "running"             */
 ml8rt_set_int_edge(IRQ_C, 1);
 ml8rt_unmask_int(IRQ_C);              /* enable interrupt                 */
 ml8rt_exit();
}


void stop(void)                        /* STOP-PROCEDURE                   */
{
 ml8rt_entry();
 ml8rt_mask_int(IRQ_C);                /* disable interrupt                */
 parameter.status = STOPPED;           /* set status "stopped"             */
 ml8rt_exit();
}


void main_task(void)                   /* MAIN-PROCEDURE OF THE TASK       */
{
 unsigned int i, sccadr;


 ml8rt_entry();

 sccadr = mba[parameter.modul] + channels[parameter.scc_channel];

 ml8rt_local_led_on();

 m048_scc_intak(parameter.modul, &parameter.int_vector, &parameter.rr0,
                &parameter.rr1, &parameter.rr3);


				       /* clear interrupting condition:    */

				       /* first the user must determine    */
				       /* the interrupt source.            */
				       /* vector=scc-number&status V1-V3   */
				       /* rr0   =extended status info      */
				       /* rr1   =special rec. cond. status */
				       /* rr3   =interrupt pending reg.    */

				       /* here :                           */
				       /* reset ext./status interrupts     */
 outportb(sccadr, 0x10);

				       /* reset highest IUS                */
 outportb(sccadr, 0x38);

 ml8rt_local_led_off();

 outportb(MASTER_INT_CONTR, 0x20);     /* EOI to Master-Controller   	   */

 ml8rt_exit_interrupt();
}


void init_pdt(void)                    /* PDT-INIT                         */
{
 pdt.type        = 1;                  /* set PDT-type                     */
 pdt.lengt       = _PDT_LENGTH;        /* length of PDT-header             */
 pdt.proc_num    = 4;                  /* number of procedures             */
 pdt.prg_number  = PRG_NUMBER;         /* program-number  		   */
 pdt.version     = VERSION;            /* program-version 		   */
 pdt.revision    = REVISION;           /* programm-revision 		   */
 pdt.cpu_type    = _486DX;             /* CPU-type 			   */
 pdt.coproz_type = _487DX;             /* Co-Prozessor-type        	   */
 pdt.language    = _CPP31;             /* program-language = C++           */
 pdt.prg_type    = _USER_PRG;          /* program-type: user-program       */
 pdt.flags       = 0;                  /* flags                            */
 pdt.flags       = _DI_TASK +
		   _PDT_INFO_ENABLE +
		   _LOCAL_DATA +
		   _LOCAL_PARAMETER +
		   _FIXED_DATASIZE;
 pdt.interrupt_num = IRQ_C;            /* interrupt-number                 */
 pdt.data_adr    = 0;                  /* address of data-range 	   */
 pdt.data_size   = 0;                  /* size of data-range 	 	   */
 pdt.data_min    = 0;                  /* min. size of data-range	   */
 pdt.data_max    = 0;                  /* max. size of data-range          */
				       /* start address of parameter-range */
 ml8rt_phys_adr(&parameter, &pdt.parameter_adr);
 pdt.parameter_adr  = pdt.parameter_adr + sizeof(tdt_type);
				       /* number of parameter              */
 pdt.parameter_size = sizeof(parameter) - sizeof(tdt_type);
 pdt.hyper_adr  = 0;                   /* address of hypertext-range 	   */
 pdt.main_proc  = main_task;           /* address of main procedure 	   */
 pdt.auto_init  = auto_init;           /* address of auto_init procedure   */
 pdt.start_proc = start;               /* address of start procedure  	   */
 pdt.stop_proc  = stop;                /* address of stop procedure 	   */
}



void main (void)                       /* PREPARE   			   */
{
 init_pdt();                           /* PDT init.  			   */
 ml8rt_set_pdt_adr(&pdt);              /* return address of PDT to OS      */
}
