PROGRAM M8P0330;

(***************************************************************************)
(**            M-COM-8 string transmit (NI-Task, MODULAR-4/486)           **)
(**                                                                       **)
(**  This program (NI-Task) transmits a string using module M-COM-8.      **)
(**  It uses the module-library M048_LIB.                                 **)
(**  Please note, that the EEPROM-contents are used to program the module.**)
(**                                                                       **)
(**                                                                       **)
(**                                                                       **)
(**  Installation Reference:                                              **)
(**                                                                       **)
(**  program-number   : 330h                                              **)
(**  task-number      : defined by user                                   **)
(**  interrupt-source : 1 (not used, NI-Task)                             **)
(**  data-range       : 0                                                 **)
(**  flags            : 0980h (don't activate, call Auto-init, NI-Task)   **)
(**                                                                       **)
(**                                                                       **)
(**                                                                       **)
(**                     for Borland Turbo Pascal 7.0                      **)
(**                                                                       **)
(**                             Version 1.A.                              **)
(**                                                                       **)
(**                               date : 07/26/95                         **)
(**                        last change : 08/21/95                         **)
(**                             status : not released                     **)
(**                             author : A.H.                             **)
(**                                                                       **)
(**                (c) 1995 SORCUS Computer GmbH Heidelberg               **)
(**                                                                       **)
(***************************************************************************)
{$F+}                                  (* create far calls                 *)
{$N+}                                  (* use coprocessor                  *)
{$E+}                                  (* no emulation                     *)
{$R-}                                  (* no range checking                *)
{$S-}                                  (* no stack checking                *)
{$I-}                                  (* no i/o checking                  *)
{$V-}                                  (* string as var                    *)
{$M 1024, 0, 0}                        (* stack and heap = minimum         *)

USES ml8rtbib,                         (* header of real-time library      *)
     m048_lib;                         (* header of module library         *)


CONST				       (* general defines                  *)
     PRG_NUMBER: Word = $330;          (* program-number := 330h           *)
     VERSION   : Char = '1';           (* program-version                  *)
     REVISION  : Char = 'A';           (* program-revision                 *)

				       (* status-definitions of task       *)
     READY     = 0;                    (* program ready                    *)
     RUNNING   = 2;                    (* programm active                  *)
     ERROR     = 3;                    (* error                            *)
     STOPPED   = 4;                    (* program halted                   *)

     MODUL     = 3;                    (* 1..4..10                         *)

     SCC_CHANNEL    = 7;               (* 0 := A , 1 := B, ..., 7 := H     *)
     SCC_MODE       = $56;             (* clock mode := master             *)
     SCC_INTMODE    = $00;             (* no interrupts                    *)
     SCC_INT_X_MODE = $00;             (* no ext. interrupts               *)
     SCC_QUARTZ     = 25000000;        (* frequence of clk-oszillator      *)
     SCC_BAUDRATE   = 19200;           (* desired baudrate                 *)
     SCC_DATABITS   = 8;               (* number of databits (5..8)        *)
     SCC_STOPBITS   = 1;               (* 0=0 stopbits, 1=1, 2=1,5, 3=2    *)
     SCC_PARTYPE    = 0;               (* 0=no parity, 1=odd, 2=even       *)


     Text : string = 'Transmitting Data using M-COM-8 !' + chr(13) + chr(13);



     MBA  : ARRAY[0..10] OF Word = (   0, $400, $500, $600, $700, $420, $440,
                                    $460, $480, $4A0, $4C0);


				       (* Program-Descriptor-Table (PDT)   *)
Type PDT_Type = Record
                Typ       : Byte;      (* type of PDT                      *)
                Length    : Byte;      (* length of PDT-header             *)
                Proc_num  : Word;      (* number of procedures             *)
                Prg_number: Word;      (* program-number                   *)
                Version   : Char;      (* program-version                  *)
                Revision  : Char;      (* program revision                 *)

                Cpu_type  : Byte;      (* processor-type                   *)
                Copro_type: Byte;      (* co-prozessor-type                *)
                Language  : Byte;      (* programming language             *)
                Prg_type  : Byte;      (* program-type                     *)

                Flags     : Word;      (* installation-flags               *)
                Interrupt_num: Word;   (* number of the used interrupt     *)

                Data_adr  : Longint;   (* start-address of data-range      *)
                Data_size : Longint;   (* size of data-range               *)
                Data_min  : Longint;   (* minimum size of data-range       *)
                Data_max  : Longint;   (* maximum size of data-range       *)

                Parameter_adr: Longint;(* address of parameter-range       *)
                Parameter_size: Word;  (* number of parameters            *)

                Hyper_adr: Longint;    (* start-address of hypertext-range *)
                Res1: Word;
                Res2: Longint;

                Main_Proc : Pointer;   (* address of Main-procedure        *)
                Auto_Init : Pointer;   (* address of Auto-Init procedure   *)
                Start_Proc: Pointer;   (* address of Start-procedure       *)
                Stop_Proc : Pointer;   (* address of Stop-procedure        *)
                END;


VAR PDT : PDT_Type;

    Parameter: Record   	       (* definition of parameter-range    *)
               TDT   : TDT_Type;       (* Task-Descriptor-Table TDT        *)
               Status: Byte;           (* actual status of program         *)
               END;

    i        : Integer;                (* Transmitted Byte Number...       *)


PROCEDURE Init_Module;
VAR
 timeconst: Word;
 basis_reg, clkmode, use_brg: Byte;
 baud_error: Longint;

BEGIN

 m048_bib_startup(MODUL);              (* start library                    *)

				       (* reset scc                        *)
 m048_scc_reset(MODUL, (SCC_CHANNEL DIV 2));

				       (* calculate baudrate-parameters    *)
 m048_baudrate_calc(SCC_BAUDRATE, SCC_QUARTZ, baud_error,
		    basis_reg, clkmode, timeconst, use_brg);
                                       (* set baseclk1-register            *)
 port[MBA[MODUL] + $12] := basis_reg;
				       (* set channel/clksel-register      *)
 port[MBA[MODUL] + $10] := SCC_CHANNEL;
 port[MBA[MODUL] + $11] := $01;        (* set to baseclk1                  *)

				       (* SCC-init.                        *)
 m048_init_scc(MODUL, SCC_CHANNEL, SCC_MODE, SCC_INTMODE, SCC_INT_X_MODE,
	       timeconst, use_brg, clkmode, SCC_DATABITS, SCC_STOPBITS, SCC_PARTYPE);

END;


PROCEDURE Auto_Init;                   (* AUTO-INIT                        *)
BEGIN
 ml8rt_entry;                          (* save registers and prepare       *)

 ml8rt_local_led_off;                  (* on-board-LED off                 *)

 ml8rt_exit;                           (* restore registers                *)
END;


PROCEDURE Start;                       (* START-PROCEDURE                  *)
BEGIN
 ml8rt_entry;                          (* save registers and prepare       *)

 ml8rt_local_led_off;

 Init_Module;

 parameter.status := RUNNING;          (* set status "running"             *)

 i := 1;

 ml8rt_wakeup_task(parameter.tdt.task);(* enable task                      *)


 ml8rt_exit;                           (* restore registers                *)
END;


PROCEDURE Stop;                        (* STOP-PROCEDURE                   *)
BEGIN
 ml8rt_entry;                          (* save registers and prepare       *)

 ml8rt_sleep_task(parameter.tdt.task); (* disable task                     *)
 parameter.status := STOPPED;          (* set status "stopped"             *)

 ml8rt_exit;
END;


PROCEDURE Main_Task;                   (* MAIN-PROCEDURE OF THE TASK       *)
VAR
 tbe: Byte;
BEGIN

 ml8rt_entry;                          (* save registers and prepare       *)

 ml8rt_local_led_on;

 IF(i < 39) THEN
   BEGIN                               (* try to transmit character        *)
    tbe := m048_transmit_scc_character(MODUL, SCC_CHANNEL, Byte(Text[i]));
    IF(tbe = 0) THEN INC(i);           (* if successful, increment pointer *)
   END
 ELSE ml8rt_local_led_off;             (* string transmitted               *)

 ml8rt_exit;                           (* restore registers                *)
END;


PROCEDURE Init_PDT;                    (* PDT-INIT                         *)
BEGIN
 PDT.typ         := 1;                 (* set PDT-type                     *)
 PDT.length      := _PDT_LENGTH;       (* length of PDT-header             *)
 PDT.proc_num    := 4;                 (* number of procedures             *)
 PDT.prg_number  := PRG_NUMBER;        (* program-number                   *)
 PDT.version     := VERSION;           (* program-version                  *)
 PDT.revision    := REVISION;          (* programm-revision                *)
 PDT.cpu_type    := _486DX;            (* CPU-type                         *)
 PDT.copro_type  := _487DX;            (* Co-Prozessor-type                *)
 PDT.language    := _TP70;            (* program-language := C++           *)
 PDT.prg_type    := _USER_PRG;         (* program-type: user-program       *)
 PDT.flags       := 0;                 (* flags                            *)
 PDT.flags       := _NI_TASK +
		   _PDT_INFO_ENABLE +
		   _LOCAL_DATA +
		   _LOCAL_PARAMETER +
		   _FIXED_DATASIZE;
 PDT.interrupt_num := 0;               (* interrupt-number                 *)
 PDT.data_adr    := 0;                 (* address of data-range            *)
 PDT.data_size   := 0;                 (* size of data-range               *)
 PDT.data_min    := 0;                 (* min. size of data-range          *)
 PDT.data_max    := 0;                 (* max. size of data-range          *)
				       (* start address of parameter-range *)
 ml8rt_phys_adr(@parameter, PDT.parameter_adr);
 PDT.parameter_adr  := PDT.parameter_adr + sizeof(tdt_type);
				       (* number of parameter              *)
 PDT.parameter_size := Sizeof(parameter) - Sizeof(tdt_type);
 PDT.hyper_adr  := 0;                  (* address of hypertext-range       *)
 PDT.main_proc  := @Main_Task;         (* address of main procedure        *)
 PDT.auto_init  := @Auto_Init;         (* address of auto_init procedure   *)
 PDT.start_proc := @Start;             (* address of start procedure       *)
 PDT.stop_proc  := @Stop;              (* address of stop procedure        *)
END;



BEGIN                                  (* PREPARE                          *)
 init_PDT;                             (* PDT init.                        *)
 ml8rt_set_PDT_adr(PDT);               (* return address of PDT to OS      *)
END.


